from unittest import TestCase
from .consulta import ConsultaNFe, RetornoConsulta, RetornoNFe
from pathlib import Path
from lxml.etree import fromstring
from .cliente import ClienteMock
from abstra_notas.assinatura import AssinadorMock
from abstra_notas.validacoes.xml_iguais import assert_xml_iguais
from datetime import date


class ConsultaTest(TestCase):
    def test_exemplo(self):
        assinador = AssinadorMock()
        self.maxDiff = None
        exemplo_path = Path(__file__).parent / "exemplos" / "PedidoConsultaNFe.xml"
        exemplo_xml = assinador.assinar_xml(fromstring(exemplo_path.read_bytes()))

        pedido = ConsultaNFe(
            remetente="99999997000100",
            chave_nfe_inscricao_prestador="39616924",
            chave_nfe_numero_nfe=17943,
            chave_rps_inscricao_prestador="39616924",
            chave_rps_serie_rps="BB",
            chave_rps_numero_rps=4106,
        )

        pedido_xml = assinador.assinar_xml(pedido.gerar_xml(assinador=assinador))
        assert_xml_iguais(
            pedido_xml, exemplo_xml, ignorar_tags=["Assinatura", "Signature"]
        )

        cliente = ClienteMock()
        resultado = cliente.consultar_nota(pedido)
        self.maxDiff = None
        self.assertEqual(
            resultado,
            RetornoConsulta(
                lista_nfe=[
                    RetornoNFe(
                        assinatura="d8Pg/jdA7t5tSaB8Il1d/CMiLGgfFAXzTL9o5stv6TNbhm9I94DIo0/ocqJpGx0KzoEeIQz4RSn99pWX4fiW/aETlNT3u5woqCAyL6U2hSyl/eQfWRYrqFu2zcdc4rsAG/wJbDjNO8y0Pz9b6rlTwkIJ+kMdLo+EWXMnB744olYE721g2O9CmUTvjtBgCfVUgvuN1MGjgzpgyussCOSkLpGbrqtM5+pYMXZsTaEVIIck1baDkoRpLmZ5Y/mcn1/Om1fMyhJVUAkgI5xBrORuotIP7e3+HLJnKgzQQPWCtLyEEyAqUk9Gq64wMayITua5FodaJsX+Eic/ie3kS5m50Q==",
                        chave_nfe_inscricao_prestador="39616924",
                        chave_nfe_numero_nfe=17943,
                        chave_nfe_codigo_verificacao="PH5GL6XU",
                        data_emissao_nfe=date(2015, 1, 28),
                        chave_rps_inscricao_prestador="39616924",
                        chave_rps_serie_rps="BB",
                        chave_rps_numero_rps=4105,
                        tipo_rps="RPS-M",
                        data_emissao_rps=date(2015, 1, 20),
                        cpf_cnpj_prestador="99999997000100",
                        razao_social_prestador="JVA LAVANDERIA LTDA",
                        tipo_logradouro_prestador="R",
                        logradouro_prestador="PEDRO AMERICO",
                        numero_endereco_prestador="00032",
                        complemento_endereco_prestador="27 ANDAR",
                        bairro_prestador="CENTRO",
                        cidade_prestador="3550308",
                        uf_prestador="SP",
                        cep_prestador="1045010",
                        status_nfe="N",
                        tributacao_nfe="T",
                        opcao_simples=4,
                        valor_servicos_centavos=20500_00,
                        valor_deducoes_centavos=5000_00,
                        valor_pis_centavos=10_00,
                        valor_cofins_centavos=10_00,
                        valor_inss_centavos=10_00,
                        valor_ir_centavos=10_00,
                        valor_csll_centavos=10_00,
                        valor_iss_centavos=0,
                        valor_credito_centavos=139_50,
                        codigo_servico=7617,
                        aliquota_servicos=0.0,
                        iss_retido=False,
                        cpf_cnpj_tomador="12345678909",
                        razao_social_tomador="JOAO TESTE",
                        tipo_logradouro_tomador="R",
                        logradouro_tomador="Sao Bento",
                        numero_endereco_tomador="100",
                        bairro_tomador="Centro",
                        cidade_tomador="3550308",
                        uf_tomador="SP",
                        cep_tomador="1010000",
                        email_tomador="teste@teste.com.br",
                        discriminacao="Desenvolvimento de Web Site Pessoal.",
                        fonte_carga_tributaria=None,
                    ),
                    RetornoNFe(
                        assinatura="XwAGUeJfR3AVRctSPTMyuF0wzkd5LSbsOpFEXxVCLdMI4RP7lGEAYwoHFSCfoL6iYH/XMGAXWpua8YUSjPzAXC//TpFnoaNmpN5f9YvpxhW/0y4NiZfYTBiuLlArDLXSG7691vMixk6xRnAXY1eL4GdgrW8GU8VtLjp1tyku5u+gzfkCJDaLi1h2JplbCCvXtNr7smcJl7srYo/MuyLvuawEBCWPr3TtLtRyEu3fsOX8E3TK6ReF5kJ1BF+lO6SIX0GFw+OxQukPVeJt3WXDGZseV8Uh7GNBgxpTRIf1LU4fSoIDXGDzE0UgNJWRKEgcMbDEECBTCYNHl5vFdmePxA==",
                        chave_nfe_inscricao_prestador="39616924",
                        chave_nfe_numero_nfe=17944,
                        chave_nfe_codigo_verificacao="RAJXWXJP",
                        data_emissao_nfe=date(2015, 1, 28),
                        chave_rps_inscricao_prestador="39616924",
                        chave_rps_serie_rps="BB",
                        chave_rps_numero_rps=4106,
                        tipo_rps="RPS-M",
                        data_emissao_rps=date(2015, 1, 20),
                        cpf_cnpj_prestador="99999997000100",
                        razao_social_prestador="JVA LAVANDERIA LTDA",
                        tipo_logradouro_prestador="R",
                        logradouro_prestador="PEDRO AMERICO",
                        numero_endereco_prestador="00032",
                        complemento_endereco_prestador="27 ANDAR",
                        bairro_prestador="CENTRO",
                        cidade_prestador="3550308",
                        uf_prestador="SP",
                        cep_prestador="1045010",
                        status_nfe="N",
                        tributacao_nfe="T",
                        opcao_simples=4,
                        valor_servicos_centavos=20501_00,
                        valor_deducoes_centavos=5000_00,
                        valor_pis_centavos=10_00,
                        valor_cofins_centavos=10_00,
                        valor_inss_centavos=10_00,
                        valor_ir_centavos=10_00,
                        valor_csll_centavos=10_00,
                        valor_credito_centavos=139_50,
                        valor_iss_centavos=0,
                        codigo_servico=7617,
                        aliquota_servicos=0.0,
                        iss_retido=False,
                        cpf_cnpj_tomador="12345678909",
                        razao_social_tomador="JOAO TESTE",
                        tipo_logradouro_tomador="R",
                        logradouro_tomador="Sao Bento",
                        numero_endereco_tomador="100",
                        bairro_tomador="Centro",
                        cidade_tomador="3550308",
                        uf_tomador="SP",
                        cep_tomador="1010000",
                        email_tomador="teste@teste.com.br",
                        discriminacao="Desenvolvimento de Web Site Pessoal.",
                        fonte_carga_tributaria=None,
                    ),
                ]
            ),
        )



from enum import Enum


class TipoLogradouro(Enum):
    AREA = "A"
    ACESSO = "AC"
    ACAMPAMENTO = "ACA"
    ACESSO_LOCAL = "ACL"
    ADRO = "AD"
    AREA_ESPECIAL = "AE"
    AEROPORTO = "AER"
    ALAMEDA = "AL"
    AVENIDA_MARGINAL_DIREITA = "AMD"
    AVENIDA_MARGINAL_ESQUERDA = "AME"
    ANEL_VIARIO = "AN"
    ANTIGA_ESTRADA = "ANT"
    ARTERIA = "ART"
    ALTO = "AT"
    ATALHO = "ATL"
    AREA_VERDE = "A V"
    AVENIDA = "AV"
    AVENIDA_CONTORNO = "AVC"
    AVENIDA_MARGINAL = "AVM"
    AVENIDA_VELHA = "AVV"
    BALNEARIO = "BAL"
    BECO = "BC"
    BURACO = "BCO"
    BELVEDERE = "BEL"
    BLOCO = "BL"
    BALO = "BLO"
    BLOCOS = "BLS"
    BULEVAR = "BLV"
    BOSQUE = "BSQ"
    BOULEVARD = "BVD"
    BAIXA = "BX"
    CAIS = "C"
    CALCADA = "CAL"
    CAMINHO = "CAM"
    CANAL = "CAN"
    CHACARA = "CH"
    CHAPADAO = "CHA"
    CICLOVIA = "CIC"
    CIRCULAR = "CIR"
    CONJUNTO = "CJ"
    CONJUNTO_MUTIRAO = "CJM"
    COMPLEXO_VIARIO = "CMP"
    COLONIA = "COL"
    COMUNIDADE = "COM"
    CONDOMINIO = "CON"
    CORREDOR = "COR"
    CAMPO = "CPO"
    CORREGO = "CRG"
    CONTORNO = "CTN"
    DESCIDA = "DSC"
    DESVIO = "DSV"
    DISTRITO = "DT"
    ENTRE_BLOCO = "EB"
    ESTRADA_INTERMUNICIPAL = "EIM"
    ENSEADA = "ENS"
    ENTRADA_PARTICULAR = "ENT"
    ENTRE_QUADRA = "EQ"
    ESCADA = "ESC"
    ESCADARIA = "ESD"
    ESTRADA_ESTADUAL = "ESE"
    ESTRADA_VICINAL = "ESI"
    ESTRADA_DE_LIGACAO = "ESL"
    ESTRADA_MUNICIPAL = "ESM"
    ESPLANADA = "ESP"
    ESTRADA_DE_SERVIDAO = "ESS"
    ESTRADA = "EST"
    ESTRADA_VELHA = "ESV"
    ESTRADA_ANTIGA = "ETA"
    ESTACAO = "ETC"
    ESTADIO = "ETD"
    ESTANCIA = "ETN"
    ESTRADA_PARTICULAR = "ETP"
    ESTACIONAMENTO = "ETT"
    EVANGELICA = "EVA"
    ELEVADA = "EVD"
    EIXO_INDUSTRIAL = "EX"
    FAVELA = "FAV"
    FAZENDA = "FAZ"
    FERROVIA = "FER"
    FONTE = "FNT"
    FEIRA = "FRA"
    FORTE = "FTE"
    GALERIA = "GAL"
    GRANJA = "GJA"
    NUCLEO_HABITACIONAL = "HAB"
    ILHA = "IA"
    INDETERMINADO = "IND"
    ILHOTA = "IOA"
    JARDIM = "JD"
    JARDINETE = "JDE"
    LADEIRA = "LD"
    LAGOA = "LGA"
    LAGO = "LGO"
    LOTEAMENTO = "LOT"
    LARGO = "LRG"
    LOTE = "LT"
    MERCADO = "MER"
    MARINA = "MNA"
    MODULO = "MOD"
    PROJECAO = "MRG"
    MORRO = "MRO"
    MONTE = "MTE"
    NUCLEO = "NUC"
    NUCLEO_RURAL = "NUR"
    OUTEIRO = "OUT"
    PARALELA = "PAR"
    PASSEIO = "PAS"
    PATIO = "PAT"
    PRACA = "PC"
    PRACA_DE_ESPORTES = "PCE"
    PARADA = "PDA"
    PARADOURO = "PDO"
    PONTA = "PNT"
    PRAIA = "PR"
    PROLONGAMENTO = "PRL"
    PARQUE_MUNICIPAL = "PRM"
    PARQUE = "PRQ"
    PARQUE_RESIDENCIAL = "PRR"
    PASSARELA = "PSA"
    PASSAGEM = "PSG"
    PASSAGEM_DE_PEDESTRE = "PSP"
    PASSAGEM_SUBTERRANEA = "PSS"
    PONTE = "PTE"
    PORTO = "PTO"
    QUADRA = "Q"
    QUINTA = "QTA"
    QUINTAS = "QTS"
    RUA = "R"
    RUA_INTEGRACAO = "R I"
    RUA_DE_LIGACAO = "R L"
    RUA_PARTICULAR = "R P"
    RUA_VELHA = "R V"
    RAMAL = "RAM"
    RECREIO = "RCR"
    RECANTO = "REC"
    RETIRO = "RER"
    RESIDENCIAL = "RES"
    RETA = "RET"
    RUELA = "RLA"
    RAMPA = "RMP"
    RODO_ANEL = "ROA"
    RODOVIA = "ROD"
    ROTULA = "ROT"
    RUA_DE_PEDESTRE = "RPE"
    MARGEM = "RPR"
    RETORNO = "RTN"
    ROTATORIA = "RTT"
    SEGUNDA_AVENIDA = "SEG"
    SITIO = "SIT"
    SERVIDAO = "SRV"
    SETOR = "ST"
    SUBIDA = "SUB"
    TRINCHEIRA = "TCH"
    TERMINAL = "TER"
    TRECHO = "TR"
    TREVO = "TRV"
    TUNEL = "TUN"
    TRAVESSA = "TV"
    TRAVESSA_PARTICULAR = "TVP"
    TRAVESSA_VELHA = "TVV"
    UNIDADE = "UNI"
    VIA = "V"
    VIA_COLETORA = "V C"
    VIA_LOCAL = "V L"
    VIA_DE_ACESSO = "VAC"
    VALA = "VAL"
    VIA_COSTEIRA = "VCO"
    VIADUTO = "VD"
    VIA_EXPRESSA = "V-E"
    VEREDA = "VER"
    VIA_ELEVADO = "VEV"
    VILA = "VL"
    VIELA = "VLA"
    VALE = "VLE"
    VIA_LITORANEA = "VLT"
    VIA_DE_PEDESTRE = "VPE"
    VARIANTE = "VRT"
    ZIGUE_ZAGUE = "ZIG"



from unittest import TestCase
from .cpf import normalizar_cpf, CpfInvalido


class TestCPF(TestCase):
    def test_remove_formatação(self):
        cpf = normalizar_cpf("083.941.150-20")
        self.assertEqual(cpf, "08394115020")

    def test_aceita_sem_formatação(self):
        cpf = normalizar_cpf("08394115020")
        self.assertEqual(cpf, "08394115020")

    def test_error_quantidade_de_caracteres(self):
        with self.assertRaises(CpfInvalido):
            normalizar_cpf("123")



from jinja2 import Template
from pathlib import Path

def load_template(template_name: str) -> Template:
    """
    Load a Jinja2 template by its name.

    :param template_name: The name of the template to load.
    :return: A Jinja2 Template object.
    """

    return Template((Path(__file__).parent / f"{template_name}.xml").read_text(encoding="utf-8"))



from dateutil.parser import parse
from datetime import date


def normalizar_data(data, optional=False):
    if data is None and optional:
        return None

    if isinstance(data, str):
        return parse(data).date()
    elif isinstance(data, date):
        return data
    else:
        raise ValueError(f"Data inválida. {repr(data)}")



from unittest import TestCase
from .cnpj import normalizar_cnpj, CnpjInvalido


class TestCNPJ(TestCase):
    def test_remove_formatação(self):
        cnpj = normalizar_cnpj("02.981.391/0001-06")
        self.assertEqual(cnpj, "02981391000106")

    def test_aceita_sem_formatação(self):
        cnpj = normalizar_cnpj("02981391000106")
        self.assertEqual(cnpj, "02981391000106")

    def test_error_quantidade_de_caracteres(self):
        with self.assertRaises(CnpjInvalido):
            normalizar_cnpj("123")
